class FrequencyWeighting(Layer):
    """

    The computation code is from librosa.
    Brian, thanks x NaN!

    """

    def __init__(self, mode, frequencies, decibel, power, **kwargs):
        """
        mode: 'A' 'a' for A-weighting
        frequencies: list of float or 1d np array, center frequencies.
        decibel: Bool, true if input is decibel scale (log(X))
        power: float but probably either 1.0 or 2.0.
        E.g., if input is power spectrogram which is log(X**2),
            power = 2.0,
            decibel = True
        If decibel:
            in call(), weights are ADDED.
        else:
            in call(), weights are MULTIPLIED.

        """
        # TODO: Current code is for keras v1.
        assert mode.lower() in ('a')
        self.mode = mode
        self.frequencies = frequencies
        self.decibel = decibel
        self.power = power
        super(FrequencyWeighting, self).__init__(**kwargs)

        freq_weights = backend.a_weighting(self.frequencies)

        if power != 2.0:
            freq_weights *= (power / 2.0)
        if decibel:
            self.freq_weights = K.variable(freq_weights, dtype=K.floatx())
        else:
            self.freq_weights = K.variable(10. ** freq_weights, dtype=K.floatx())

    def call(self, x, mask=None):
        if self.decibel:
            return x + self.freq_weights
        else:
            return x * self.freq_weights

    def get_config(self):
        config = {'mode': self.mode,
                  'frequencies': self.frequencies,
                  'decibel': self.decibel}
        base_config = super(FrequencyWeighting, self).get_config()
        return dict(list(base_config.items()) + list(config.items()))



"""data downloader. Keunwoo Choi, 14 Mar 2017.
Copied from keras.utils.data_utils and modify path/api a little.
99.999% credit to Keras.
"""
from __future__ import absolute_import
from __future__ import print_function

import functools
import tarfile
import os
import sys
import shutil
# import hashlib
from six.moves.urllib.request import urlopen
from six.moves.urllib.error import URLError
from six.moves.urllib.error import HTTPError

from keras.utils.generic_utils import Progbar
from keras.utils.data_utils import validate_file

import pandas as pd

allowed_exts = set(['mp3', 'wav', 'au', 'm4a'])

if sys.version_info[0] == 2:
    def urlretrieve(url, filename, reporthook=None, data=None):
        """Replacement for `urlretrive` for Python 2.
        Under Python 2, `urlretrieve` relies on `FancyURLopener` from legacy
        `urllib` module, known to have issues with proxy management.
        # Arguments
            url: url to retrieve.
            filename: where to store the retrieved data locally.
            reporthook: a hook function that will be called once
                on establishment of the network connection and once
                after each block read thereafter.
                The hook will be passed three arguments;
                a count of blocks transferred so far,
                a block size in bytes, and the total size of the file.
            data: `data` argument passed to `urlopen`.
        """

        def chunk_read(response, chunk_size=8192, reporthook=None):
            total_size = response.info().get('Content-Length').strip()
            total_size = int(total_size)
            count = 0
            while 1:
                chunk = response.read(chunk_size)
                count += 1
                if not chunk:
                    reporthook(count, total_size, total_size)
                    break
                if reporthook:
                    reporthook(count, chunk_size, total_size)
                yield chunk

        response = urlopen(url, data)
        with open(filename, 'wb') as fd:
            for chunk in chunk_read(response, reporthook=reporthook):
                fd.write(chunk)
else:
    from six.moves.urllib.request import urlretrieve


def get_file(fname, origin, save_path, untar=False,
             md5_hash=None, cache_subdir='datasets', tar_folder_name=None):
    """Downloads a file from a URL if it not already in the cache.
    Passing the MD5 hash will verify the file after download
    as well as if it is already present in the cache.

    Usually it downloads the file to
        save_path/cache_dubdir/fname

    Arguments
    ---------
        fname: name of the file
        origin: original URL of the file
        save_path: path to create cache_subdir.
        untar: boolean, whether the file should be decompressed
        md5_hash: MD5 hash of the file for verification
        cache_subdir: directory being used as the cache
        tar_folder_name: string, if inside of abc.tar.gz is not abc but def, pass def here.

    Returns
    -------
        Path to the downloaded file
    """
    datadir_base = save_path
    if not os.access(datadir_base, os.W_OK):
        datadir_base = os.path.expanduser(os.path.join('~', '.kapre'))
        print('Given path {} is not accessible. Trying to use~/.kapre instead..')
        if not os.access(datadir_base, os.W_OK):
            print('~/.kapre is not accessible, using /tmp/kapre instead.')
            datadir_base = os.path.join('/tmp', '.kapre')
    datadir = os.path.join(datadir_base, cache_subdir)

    if not os.path.exists(datadir):
        os.makedirs(datadir)

    if untar:
        assert fname.endswith('.tar.gz'), fname
        fpath = os.path.join(datadir, fname)
        if tar_folder_name:
            untar_fpath = os.path.join(datadir, tar_folder_name)
        else:
            untar_fpath = fpath.rstrip('.tar.gz')
    else:
        fpath = os.path.join(datadir, fname)

    download = False
    if os.path.exists(fpath):
        # File found; verify integrity if a hash was provided.
        if md5_hash is not None:
            if not validate_file(fpath, md5_hash):
                print('A local file was found, just checked md5 hash, but it might be '
                      'incomplete or outdated')
                download = True
    else:
        download = True

    if download:
        print('Downloading data from', origin)
        progbar = None

        def dl_progress(count, block_size, total_size, progbar=None):
            if progbar is None:
                progbar = Progbar(total_size)
            else:
                progbar.update(count * block_size)

        error_msg = 'URL fetch failure on {}: {} -- {}'
        try:
            try:
                urlretrieve(origin, fpath,
                            functools.partial(dl_progress, progbar=progbar))
            except URLError as e:
                raise Exception(error_msg.format(origin, e.errno, e.reason))
            except HTTPError as e:
                raise Exception(error_msg.format(origin, e.code, e.msg))
        except (Exception, KeyboardInterrupt) as e:
            if os.path.exists(fpath):
                os.remove(fpath)
            raise
        progbar = None

    if untar:
        if not os.path.exists(untar_fpath):
            print('Untaring file...')
            tfile = tarfile.open(fpath, 'r:gz')
            try:
                tfile.extractall(path=datadir)
            except (Exception, KeyboardInterrupt) as e:
                if os.path.exists(untar_fpath):
                    if os.path.isfile(untar_fpath):
                        os.remove(untar_fpath)
                    else:
                        shutil.rmtree(untar_fpath)
                raise
            tfile.close()
            # return untar_fpath

    return datadir


def write_to_csv(rows, column_names, csv_fname):
    '''write a csv file using given rows, columns, and filename

    Arguments
    ---------
        rows: list of rows (= which are lists.)
        column_names: names for columns
        csv_fname: string, csv file name

    Returns
    -------
        None
    '''
    df = pd.DataFrame(rows, columns=column_names)
    df.to_csv(csv_fname)


def get_rows_from_folders(folder_dataset, folders, dataroot=''):
    '''Utility function to create a csv file for a dataset.
    For gtzan music, gtzan speech, ballroom extended, jamendo,
    where each class in different folders.

    Arguments
    ---------
        folder_dataset: folder name in the dataset
        folders: name of subfolders. Usually same as class names.
        dataroot: base path of the dataset. Default is '' in case if you just wanna
            specify the whole path in folder_dataset.

    Returns
    -------
        list of lists, [file_id, file_path, file_label] where
            file_id: just file names
            file_path: file paths of each file (without data_root for more universal usage)
            file_label: integer, 0 to N-1, according to the order in `folders`.

    Example
    -------
    folders = ['blues', 'classical', 'country', 'disco', 'hiphop', 'jazz', 'metal', 'pop', 'reggae', 'rock']
    rows = get_rows_from_folders('gtzan_music', folders, '/usr/keunwoo/datasets')
    '''
    rows = []
    for label_idx, folder in enumerate(folders):  # assumes different labels per folders.
        files = os.listdir(os.path.join(dataroot, folder_dataset, folder))
        files = [f for f in files if f.split('.')[-1].lower() in allowed_exts]
        for fname in files:
            file_path = os.path.join(folder_dataset, folder, fname)
            file_id = os.path.splitext(fname)[0]
            file_label = label_idx
            rows.append([file_id, file_path, file_label])
    return rows



# -*- coding: utf-8 -*-
from __future__ import absolute_import
import numpy as np
from keras.engine import Layer
from keras import backend as K
from . import backend
from . import backend_keras


class AmplitudeToDB(Layer):
    """

    ### `AmplitudeToDB`

    ```python
    kapre.utils.AmplitudeToDB(ref_power=1.0, amin=1e-10, top_db=80.0, **kargs)
    ```

    A layer that converts amplitude to decibel

    #### Parameters

    * ref_power: float [scalar]
        - reference power. Default: 1.0

    * amin: float [scalar]
        - Noise floor. Default: 1e-10
        
    * top_db: float [scalar]
        - Dynamic range of output. Default: 80.0

    #### Example
    Adding ``AmplitudeToDB`` after a spectrogram:
    ```python
        model.add(Spectrogram(return_decibel=False))
        model.add(AmplitudeToDB())
    ```
    , which is the same as:
    ```python
        model.add(Spectrogram(return_decibel=True))
    ```

    """

    def __init__(self, amin=1e-10, top_db=80.0, **kwargs):
        self.amin = amin
        self.top_db = top_db
        super(AmplitudeToDB, self).__init__(**kwargs)

    def call(self, x, mask=None):
        return backend_keras.amplitude_to_decibel(x, amin=self.amin, dynamic_range=self.top_db)

    def get_config(self):
        config = {'amin': self.amin,
                  'top_db': self.top_db}
        base_config = super(AmplitudeToDB, self).get_config()
        return dict(list(base_config.items()) + list(config.items()))


class Normalization2D(Layer):
    """

    ### `Normalization2D`

    `kapre.utils.Normalization2D`

    A layer that normalises input data in ``axis`` axis.

    #### Parameters

    * input_shape: tuple of ints
        - E.g., ``(None, n_ch, n_row, n_col)`` if theano.

    * str_axis: str
        - used ONLY IF ``int_axis`` is ``None``.
        - ``'batch'``, ``'data_sample'``, ``'channel'``, ``'freq'``, ``'time')``
        - Even though it is optional, actually it is recommended to use
        - ``str_axis`` over ``int_axis`` because it provides more meaningful
        - and image data format-robust interface.

    * int_axis: int
        - axis index that along which mean/std is computed.
        - `0` for per data sample, `-1` for per batch.
        - `1`, `2`, `3` for channel, row, col (if channels_first)
        - if `int_axis is None`, ``str_axis`` SHOULD BE set.

    #### Example

    A frequency-axis normalization after a spectrogram::
        ```python
        model.add(Spectrogram())
        model.add(Normalization2D(stf_axis='freq'))
        ```
    """

    def __init__(self, str_axis=None, int_axis=None, image_data_format='default',
                 eps=1e-10, **kwargs):
        assert not (int_axis is None and str_axis is None), \
            'In Normalization2D, int_axis or str_axis should be specified.'

        assert image_data_format in ('channels_first', 'channels_last', 'default'), \
            'Incorrect image_data_format: {}'.format(image_data_format)

        if image_data_format == 'default':
            self.image_data_format = K.image_data_format()
        else:
            self.image_data_format = image_data_format

        self.str_axis = str_axis
        if self.str_axis is None: # use int_axis
            self.int_axis = int_axis
        else: # use str_axis
            # warning
            if int_axis is not None:
                print('int_axis={} passed but is ignored, str_axis is used instead.'.format(int_axis))
            # do the work
            assert str_axis in ('batch', 'data_sample', 'channel', 'freq', 'time'), \
                'Incorrect str_axis: {}'.format(str_axis)
            if str_axis == 'batch':
                int_axis = -1
            else:
                if self.image_data_format == 'channels_first':
                    int_axis = ['data_sample', 'channel', 'freq', 'time'].index(str_axis)
                else:
                    int_axis = ['data_sample', 'freq', 'time', 'channel'].index(str_axis)

        assert int_axis in (-1, 0, 1, 2, 3), 'invalid int_axis: ' + str(int_axis)
        self.axis = int_axis
        self.eps = eps
        super(Normalization2D, self).__init__(**kwargs)

    def call(self, x, mask=None):
        if self.axis == -1:
            mean = K.mean(x, axis=[3, 2, 1, 0], keepdims=True)
            std = K.std(x, axis=[3, 2, 1, 0], keepdims=True)
        elif self.axis in (0, 1, 2, 3):
            all_dims = [0, 1, 2, 3]
            del all_dims[self.axis]
            mean = K.mean(x, axis=all_dims, keepdims=True)
            std = K.std(x, axis=all_dims, keepdims=True)
        return (x - mean) / (std + self.eps)

    def get_config(self):
        config = {'int_axis': self.axis,
                  'str_axis': self.str_axis,
                  'image_data_format': self.image_data_format}
        base_config = super(Normalization2D, self).get_config()
        return dict(list(base_config.items()) + list(config.items()))



# -*- coding: utf-8 -*-
from __future__ import absolute_import
import numpy as np
from keras import backend as K


def amplitude_to_decibel(x, amin=1e-10, dynamic_range=80.0):
    """[K] Convert (linear) amplitude to decibel (log10(x)).

    x: Keras tensor or variable. 

    amin: minimum amplitude. amplitude smaller than `amin` is set to this.

    dynamic_range: dynamic_range in decibel
    """
    log_spec = 10 * K.log(K.maximum(x, amin)) / np.log(10).astype(K.floatx())
    log_spec = log_spec - K.max(log_spec)  # [-?, 0]
    log_spec = K.maximum(log_spec, -1 * dynamic_range)  # [-80, 0]
    return log_spec



# -*- coding: utf-8 -*-
from __future__ import absolute_import
import numpy as np
from keras import backend as K
from keras.engine import Layer, InputSpec


class AdditiveNoise(Layer):
    """
    ### `AdditiveNoise`

    ```python
    kapre.augmentation.AdditiveNoise(power=0.1, random_gain=False, noise_type='white', **kwargs)
    ```
    Add noise to input data and output it.

    #### Parameters
    
    * power: float [scalar]
        - The power of noise. std if it's white noise.
        - Default: ``0.1``

    * random_gain: bool
        - Whether the noise gain is random or not.
        - If ``True``, gain is sampled from ``uniform(low=0.0, high=power)`` in every batch.
        - Default: ``False``

    * noise_type; str,
        - Specify the type of noise. It only supports ``'white'`` now.
        - Default: ```white```


    #### Returns

    Same shape as input data but with additional generated noise.

    """

    def __init__(self, power=0.1, random_gain=False, noise_type='white', **kwargs):
        assert noise_type in ['white']
        self.supports_masking = True
        self.power = power
        self.random_gain = random_gain
        self.noise_type = noise_type
        self.uses_learning_phase = True
        super(AdditiveNoise, self).__init__(**kwargs)

    def call(self, x):
        if self.random_gain:
            noise_x = x + K.random_normal(shape=K.shape(x),
                                          mean=0.,
                                          stddev=np.random.uniform(0.0, self.power))
        else:
            noise_x = x + K.random_normal(shape=K.shape(x),
                                          mean=0.,
                                          stddev=self.power)

        return K.in_train_phase(noise_x, x)

    def get_config(self):
        config = {'power': self.power,
                  'random_gain': self.random_gain,
                  'noise_type': self.noise_type}
        base_config = super(AdditiveNoise, self).get_config()
        return dict(list(base_config.items()) + list(config.items()))



from setuptools import setup

setup(name='kapre',
      version='0.1.3',
      description='Kapre: Keras Audio Preprocessors. Keras layers for audio pre-processing in deep learning',
      author='Keunwoo Choi',
      url='http://github.com/keunwoo/kapre/',
      download_url='http://github.com/keunwoochoi/kapre/releases',
      author_email='keunwoo.choi@qmul.ac.uk',
      license='MIT',
      packages=['kapre'],
      install_requires=[
        'keras >= 2.0.0',
        'numpy >= 1.8.0',
        'librosa >= 0.4',
        'pandas',
        'future'
      ],
      extras_require={
          'tests': ['tensorflow'],
       },
      keywords='audio music deep learning keras',
      zip_safe=False)



from theano import tensor as T
from theano.tensor import fft


def rfft(x):
    """x: batch of data (TODO: check it out)"""
    return fft.rfft(x)
    

if __name__ == '__main__':
    N = 32

    pass